@@ -339,7 +339,7 @@ int BKE_text_reload(Text *text)
 	char str[FILE_MAX];
 	BLI_stat_t st;
 
-	if (!text || !text->name) return 0;
+	if (!text->name) return 0;
 	
 	BLI_strncpy(str, text->name, FILE_MAX);
 	BLI_path_abs(str, G.main->name);
@@ -369,14 +369,18 @@ int BKE_text_reload(Text *text)
 
 
 	buffer = MEM_mallocN(len, "text_buffer");
-	// under windows fread can return less then len bytes because
-	// of CR stripping
+	/* under windows fread can return less than len bytes because
+	 * of CR stripping */
 	len = fread(buffer, 1, len, fp);
 
 	fclose(fp);
 
-	BLI_stat(str, &st);
-	text->mtime = st.st_mtime;
+	if (BLI_stat(str, &st) != -1) {
+		text->mtime = st.st_mtime;
+	}
+	else {
+		text->mtime = 0;
+	}
 
 	text_from_buf(text, buffer, len);
 
@@ -425,14 +429,18 @@ Text *BKE_text_load_ex(Main *bmain, const char *file, const char *relpath, const
 	fseek(fp, 0L, SEEK_SET);
 	
 	buffer = MEM_mallocN(len, "text_buffer");
-	// under windows fread can return less then len bytes because
-	// of CR stripping
+	/* under windows fread can return less than len bytes because
+	 * of CR stripping */
 	len = fread(buffer, 1, len, fp);
 
 	fclose(fp);
 
-	BLI_stat(str, &st);
-	ta->mtime = st.st_mtime;
+	if (BLI_stat(str, &st) != -1) {
+		ta->mtime = st.st_mtime;
+	}
+	else {
+		ta->mtime = 0;
+	}
 	
 	text_from_buf(ta, buffer, len);
 	
@@ -446,7 +454,7 @@ Text *BKE_text_load(Main *bmain, const char *file, const char *relpath)
 	return BKE_text_load_ex(bmain, file, relpath, false);
 }
 
-Text *BKE_text_copy(Text *ta)
+Text *BKE_text_copy(Main *bmain, Text *ta)
 {
 	Text *tan;
 	TextLine *line, *tmp;
@@ -455,8 +463,7 @@ Text *BKE_text_copy(Text *ta)
 	
 	/* file name can be NULL */
 	if (ta->name) {
-		tan->name = MEM_mallocN(strlen(ta->name) + 1, "text_name");
-		strcpy(tan->name, ta->name);
+		tan->name = BLI_strdup(ta->name);
 	}
 	else {
 		tan->name = NULL;
@@ -490,6 +497,10 @@ Text *BKE_text_copy(Text *ta)
 
 	init_undo_text(tan);
 
+	if (ta->id.lib) {
+		BKE_id_lib_local_paths(bmain, ta->id.lib, &tan->id);
+	}
+
 	return tan;
 }
 
@@ -505,10 +516,14 @@ void BKE_text_unlink(Main *bmain, Text *text)
 	bNodeTree *ntree;
 	bNode *node;
 	Material *mat;
+	Lamp *la;
+	Tex *te;
+	World *wo;
+	FreestyleLineStyle *linestyle;
 	Scene *sce;
 	SceneRenderLayer *srl;
 	FreestyleModuleConfig *module;
-	short update;
+	bool update;
 
 	for (ob = bmain->object.first; ob; ob = ob->id.next) {
 		/* game controllers */
@@ -560,23 +575,97 @@ void BKE_text_unlink(Main *bmain, Text *text)
 	}
 	
 	/* nodes */
+	for (la = bmain->lamp.first; la; la = la->id.next) {
+		ntree = la->nodetree;
+		if (!ntree)
+			continue;
+		for (node = ntree->nodes.first; node; node = node->next) {
+			if (node->type == NODE_FRAME) {
+				if ((Text *)node->id == text) {
+					node->id = NULL;
+				}
+			}
+		}
+	}
+
+	for (linestyle = bmain->linestyle.first; linestyle; linestyle = linestyle->id.next) {
+		ntree = linestyle->nodetree;
+		if (!ntree)
+			continue;
+		for (node = ntree->nodes.first; node; node = node->next) {
+			if (node->type == NODE_FRAME) {
+				if ((Text *)node->id == text) {
+					node->id = NULL;
+				}
+			}
+		}
+	}
+
 	for (mat = bmain->mat.first; mat; mat = mat->id.next) {
 		ntree = mat->nodetree;
 		if (!ntree)
 			continue;
 		for (node = ntree->nodes.first; node; node = node->next) {
-			if (node->type == SH_NODE_SCRIPT) {
+			if (ELEM(node->type, SH_NODE_SCRIPT, NODE_FRAME)) {
+				if ((Text *)node->id == text) {
+					node->id = NULL;
+				}
+			}
+		}
+	}
+
+	for (te = bmain->tex.first; te; te = te->id.next) {
+		ntree = te->nodetree;
+		if (!ntree)
+			continue;
+		for (node = ntree->nodes.first; node; node = node->next) {
+			if (node->type == NODE_FRAME) {
+				if ((Text *)node->id == text) {
+					node->id = NULL;
+				}
+			}
+		}
+	}
+
+	for (wo = bmain->world.first; wo; wo = wo->id.next) {
+		ntree = wo->nodetree;
+		if (!ntree)
+			continue;
+		for (node = ntree->nodes.first; node; node = node->next) {
+			if (node->type == NODE_FRAME) {
+				if ((Text *)node->id == text) {
+					node->id = NULL;
+				}
+			}
+		}
+	}
+
+	for (sce = bmain->scene.first; sce; sce = sce->id.next) {
+		ntree = sce->nodetree;
+		if (!ntree)
+			continue;
+		for (node = ntree->nodes.first; node; node = node->next) {
+			if (node->type == NODE_FRAME) {
 				Text *ntext = (Text *)node->id;
 				if (ntext == text) node->id = NULL;
 			}
 		}
+
+		/* Freestyle (while looping oer the scene) */
+		for (srl = sce->r.layers.first; srl; srl = srl->next) {
+			for (module = srl->freestyleConfig.modules.first; module; module = module->next) {
+				if (module->script == text)
+					module->script = NULL;
+			}
+		}
 	}
-	
+
 	for (ntree = bmain->nodetree.first; ntree; ntree = ntree->id.next) {
 		for (node = ntree->nodes.first; node; node = node->next) {
-			if (node->type == SH_NODE_SCRIPT) {
-				Text *ntext = (Text *)node->id;
-				if (ntext == text) node->id = NULL;
+			if (ELEM(node->type, SH_NODE_SCRIPT, NODE_FRAME)) {
+				if ((Text *)node->id == text) {
+					node->id = NULL;
+				}
 			}
 		}
 	}
@@ -597,16 +686,6 @@ void BKE_text_unlink(Main *bmain, Text *text)
 		}
 	}
 
-	/* Freestyle */
-	for (sce = bmain->scene.first; sce; sce = sce->id.next) {
-		for (srl = sce->r.layers.first; srl; srl = srl->next) {
-			for (module = srl->freestyleConfig.modules.first; module; module = module->next) {
-				if (module->script == text)
-					module->script = NULL;
-			}
-		}
-	}
-
 	text->id.us = 0;
 }
 
@@ -614,7 +693,7 @@ void BKE_text_clear(Text *text) /* called directly from rna */
 {
 	int oldstate;
 
-	oldstate = txt_get_undostate(  );
+	oldstate = txt_get_undostate();
 	txt_set_undostate(1);
 	txt_sel_all(text);
 	txt_delete_sel(text);
@@ -647,7 +726,7 @@ int BKE_text_file_modified_check(Text *text)
 	int result;
 	char file[FILE_MAX];
 
-	if (!text || !text->name)
+	if (!text->name)
 		return 0;
 
 	BLI_strncpy(file, text->name, FILE_MAX);
@@ -676,7 +755,7 @@ void BKE_text_file_modified_ignore(Text *text)
 	int result;
 	char file[FILE_MAX];
 
-	if (!text || !text->name) return;
+	if (!text->name) return;
 
 	BLI_strncpy(file, text->name, FILE_MAX);
 	BLI_path_abs(file, G.main->name);
@@ -742,9 +821,7 @@ static TextLine *txt_new_linen(const char *str, int n)
 void txt_clean_text(Text *text)
 {	
 	TextLine **top, **bot;
-	
-	if (!text) return;
-	
+
 	if (!text->lines.first) {
 		if (text->lines.last) text->lines.first = text->lines.last;
 		else text->lines.first = text->lines.last = txt_new_line(NULL);
@@ -883,8 +960,7 @@ void txt_move_up(Text *text, const bool sel)
 {
 	TextLine **linep;
 	int *charp;
-	
-	if (!text) return;
+
 	if (sel) txt_curs_sel(text, &linep, &charp);
 	else { txt_pop_first(text); txt_curs_cur(text, &linep, &charp); }
 	if (!*linep) return;
@@ -906,8 +982,7 @@ void txt_move_down(Text *text, const bool sel)
 {
 	TextLine **linep;
 	int *charp;
-	
-	if (!text) return;
+
 	if (sel) txt_curs_sel(text, &linep, &charp);
 	else { txt_pop_last(text); txt_curs_cur(text, &linep, &charp); }
 	if (!*linep) return;
@@ -929,8 +1004,7 @@ void txt_move_left(Text *text, const bool sel)
 	TextLine **linep;
 	int *charp;
 	int tabsize = 0, i = 0;
-	
-	if (!text) return;
+
 	if (sel) txt_curs_sel(text, &linep, &charp);
 	else { txt_pop_first(text); txt_curs_cur(text, &linep, &charp); }
 	if (!*linep) return;
@@ -974,8 +1048,7 @@ void txt_move_right(Text *text, const bool sel)
 	TextLine **linep;
 	int *charp, i;
 	bool do_tab = false;
-	
-	if (!text) return;
+
 	if (sel) txt_curs_sel(text, &linep, &charp);
 	else { txt_pop_last(text); txt_curs_cur(text, &linep, &charp); }
 	if (!*linep) return;
@@ -1016,8 +1089,7 @@ void txt_jump_left(Text *text, const bool sel, const bool use_init_step)
 {
 	TextLine **linep;
 	int *charp;
-	
-	if (!text) return;
+
 	if (sel) txt_curs_sel(text, &linep, &charp);
 	else { txt_pop_first(text); txt_curs_cur(text, &linep, &charp); }
 	if (!*linep) return;
@@ -1033,8 +1105,7 @@ void txt_jump_right(Text *text, const bool sel, const bool use_init_step)
 {
 	TextLine **linep;
 	int *charp;
-	
-	if (!text) return;
+
 	if (sel) txt_curs_sel(text, &linep, &charp);
 	else { txt_pop_last(text); txt_curs_cur(text, &linep, &charp); }
 	if (!*linep) return;
@@ -1050,8 +1121,7 @@ void txt_move_bol(Text *text, const bool sel)
 {
 	TextLine **linep;
 	int *charp;
-	
-	if (!text) return;
+
 	if (sel) txt_curs_sel(text, &linep, &charp);
 	else txt_curs_cur(text, &linep, &charp);
 	if (!*linep) return;
@@ -1065,8 +1135,7 @@ void txt_move_eol(Text *text, const bool sel)
 {
 	TextLine **linep;
 	int *charp;
-	
-	if (!text) return;
+
 	if (sel) txt_curs_sel(text, &linep, &charp);
 	else txt_curs_cur(text, &linep, &charp);
 	if (!*linep) return;
@@ -1080,8 +1149,7 @@ void txt_move_bof(Text *text, const bool sel)
 {
 	TextLine **linep;
 	int *charp;
-	
-	if (!text) return;
+
 	if (sel) txt_curs_sel(text, &linep, &charp);
 	else txt_curs_cur(text, &linep, &charp);
 	if (!*linep) return;
@@ -1096,8 +1164,7 @@ void txt_move_eof(Text *text, const bool sel)
 {
 	TextLine **linep;
 	int *charp;
-	
-	if (!text) return;
+
 	if (sel) txt_curs_sel(text, &linep, &charp);
 	else txt_curs_cur(text, &linep, &charp);
 	if (!*linep) return;
@@ -1119,8 +1186,7 @@ void txt_move_to(Text *text, unsigned int line, unsigned int ch, const bool sel)
 	TextLine **linep;
 	int *charp;
 	unsigned int i;
-	
-	if (!text) return;
+
 	if (sel) txt_curs_sel(text, &linep, &charp);
 	else txt_curs_cur(text, &linep, &charp);
 	if (!*linep) return;
@@ -1157,7 +1223,6 @@ static void txt_curs_swap(Text *text)
 
 static void txt_pop_first(Text *text)
 {
-			
 	if (txt_get_span(text->curl, text->sell) < 0 ||
 	    (text->curl == text->sell && text->curc > text->selc))
 	{
@@ -1186,7 +1251,6 @@ void txt_pop_sel(Text *text)
 
 void txt_order_cursors(Text *text, const bool reverse)
 {
-	if (!text) return;
 	if (!text->curl) return;
 	if (!text->sell) return;
 	
@@ -1216,8 +1280,7 @@ static void txt_delete_sel(Text *text)
 {
 	TextLine *tmpl;
 	char *buf;
-	
-	if (!text) return;
+
 	if (!text->curl) return;
 	if (!text->sell) return;
 
@@ -1253,8 +1316,6 @@ static void txt_delete_sel(Text *text)
 
 void txt_sel_all(Text *text)
 {
-	if (!text) return;
-
 	text->curl = text->lines.first;
 	text->curc = 0;
 	
@@ -1277,7 +1338,6 @@ void txt_sel_clear(Text *text)
 
 void txt_sel_line(Text *text)
 {
-	if (!text) return;
 	if (!text->curl) return;
 	
 	text->curc = 0;
@@ -1295,8 +1355,7 @@ char *txt_to_buf(Text *text)
 	TextLine *tmp, *linef, *linel;
 	int charf, charl;
 	char *buf;
-	
-	if (!text) return NULL;
+
 	if (!text->curl) return NULL;
 	if (!text->sell) return NULL;
 	if (!text->lines.first) return NULL;
@@ -1358,7 +1417,7 @@ int txt_find_string(Text *text, const char *findstr, int wrap, int match_case)
 	TextLine *tl, *startl;
 	const char *s = NULL;
 
-	if (!text || !text->curl || !text->sell) return 0;
+	if (!text->curl || !text->sell) return 0;
 	
 	txt_order_cursors(text, false);
 
@@ -1398,8 +1457,7 @@ char *txt_sel_to_buf(Text *text)
 	int length = 0;
 	TextLine *tmp, *linef, *linel;
 	int charf, charl;
-	
-	if (!text) return NULL;
+
 	if (!text->curl) return NULL;
 	if (!text->sell) return NULL;
 	
@@ -1480,7 +1538,6 @@ void txt_insert_buf(Text *text, const char *in_buffer)
 	TextLine *add;
 	char *buffer;
 
-	if (!text) return;
 	if (!in_buffer) return;
 
 	txt_delete_sel(text);
@@ -1537,7 +1594,7 @@ static bool max_undo_test(Text *text, int x)
 			/* XXX error("Undo limit reached, buffer cleared\n"); */
 			MEM_freeN(text->undo_buf);
 			init_undo_text(text);
-			return 0;
+			return false;
 		}
 		else {
 			void *tmp = text->undo_buf;
@@ -1548,7 +1605,7 @@ static bool max_undo_test(Text *text, int x)
 		}
 	}
 
-	return 1;
+	return true;
 }
 
 #if 0  /* UNUSED */
@@ -2362,8 +2419,7 @@ void txt_split_curline(Text *text)
 {
 	TextLine *ins;
 	char *left, *right;
-	
-	if (!text) return;
+
 	if (!text->curl) return;
 
 	txt_delete_sel(text);
@@ -2405,7 +2461,6 @@ void txt_split_curline(Text *text)
 
 static void txt_delete_line(Text *text, TextLine *line)
 {
-	if (!text) return;
 	if (!text->curl) return;
 
 	BLI_remlink(&text->lines, line);
@@ -2422,8 +2477,6 @@ static void txt_delete_line(Text *text, TextLine *line)
 static void txt_combine_lines(Text *text, TextLine *linea, TextLine *lineb)
 {
 	char *tmp, *s;
-
-	if (!text) return;
 	
 	if (!linea || !lineb) return;
 
@@ -2447,7 +2500,7 @@ void txt_duplicate_line(Text *text)
 {
 	TextLine *textline;
 	
-	if (!text || !text->curl) return;
+	if (!text->curl) return;
 	
 	if (text->curl == text->sell) {
 		textline = txt_new_line(text->curl->line);
@@ -2463,8 +2516,7 @@ void txt_duplicate_line(Text *text)
 void txt_delete_char(Text *text) 
 {
 	unsigned int c = '\n';
-	
-	if (!text) return;
+
 	if (!text->curl) return;
 
 	if (txt_has_sel(text)) { /* deleting a selection */
@@ -2501,13 +2553,13 @@ void txt_delete_word(Text *text)
 {
 	txt_jump_right(text, true, true);
 	txt_delete_sel(text);
+	txt_make_dirty(text);
 }
 
 void txt_backspace_char(Text *text)
 {
 	unsigned int c = '\n';
 	
-	if (!text) return;
 	if (!text->curl) return;
 	
 	if (txt_has_sel(text)) { /* deleting a selection */
@@ -2550,6 +2602,7 @@ void txt_backspace_word(Text *text)
 {
 	txt_jump_left(text, true, true);
 	txt_delete_sel(text);
+	txt_make_dirty(text);
 }
 
 /* Max spaces to replace a tab with, currently hardcoded to TXT_TABSIZE = 4.
@@ -2571,19 +2624,18 @@ static bool txt_add_char_intern(Text *text, unsigned int add, bool replace_tabs)
 {
 	char *tmp, ch[BLI_UTF8_MAX];
 	size_t add_len;
-	
-	if (!text) return 0;
+
 	if (!text->curl) return 0;
 
 	if (add == '\n') {
 		txt_split_curline(text);
-		return 1;
+		return true;
 	}
 	
 	/* insert spaces rather than tabs */
 	if (add == '\t' && replace_tabs) {
 		txt_convert_tab_to_spaces(text);
-		return 1;
+		return true;
 	}
 
 	txt_delete_sel(text);
@@ -2631,9 +2683,8 @@ bool txt_replace_char(Text *text, unsigned int add)
 	unsigned int del;
 	size_t del_size = 0, add_size;
 	char ch[BLI_UTF8_MAX];
-	
-	if (!text) return 0;
-	if (!text->curl) return 0;
+
+	if (!text->curl) return false;
 
 	/* If text is selected or we're at the end of the line just use txt_add_char */
 	if (text->curc == text->curl->len || txt_has_sel(text) || add == '\n') {
@@ -2672,7 +2723,7 @@ bool txt_replace_char(Text *text, unsigned int add)
 		text->curc += add_size;
 		txt_pop_sel(text);
 	}
-	return 1;
+	return true;
 }
 
 void txt_indent(Text *text)
@@ -2686,7 +2737,7 @@ void txt_indent(Text *text)
 	/* hardcoded: TXT_TABSIZE = 4 spaces: */
 	int spaceslen = TXT_TABSIZE;
 
-	if (ELEM(NULL, text, text->curl, text->sell)) {
+	if (ELEM(NULL, text->curl, text->sell)) {
 		return;
 	}
 
@@ -2753,7 +2804,7 @@ void txt_unindent(Text *text)
 	/* hardcoded: TXT_TABSIZE = 4 spaces: */
 	int spaceslen = TXT_TABSIZE;
 
-	if (ELEM(NULL, text, text->curl, text->sell)) {
+	if (ELEM(NULL, text->curl, text->sell)) {
 		return;
 	}
 
@@ -2765,7 +2816,7 @@ void txt_unindent(Text *text)
 
 	while (true) {
 		bool changed = false;
-		if (strncmp(text->curl->line, remove, indentlen) == 0) {
+		if (STREQLEN(text->curl->line, remove, indentlen)) {
 			if (num == 0)
 				unindented_first = true;
 			text->curl->len -= indentlen;
@@ -2806,8 +2857,7 @@ void txt_comment(Text *text)
 	int len, num;
 	char *tmp;
 	char add = '#';
-	
-	if (!text) return;
+
 	if (!text->curl) return;
 	if (!text->sell) return;  // Need to change this need to check if only one line is selected to more than one
 
@@ -2854,8 +2904,7 @@ void txt_uncomment(Text *text)
 {
 	int num = 0;
 	char remove = '#';
-	
-	if (!text) return;
+
 	if (!text->curl) return;
 	if (!text->sell) return;
 
@@ -2901,7 +2950,7 @@ void txt_move_lines(struct Text *text, const int direction)
 
 	BLI_assert(ELEM(direction, TXT_MOVE_LINE_UP, TXT_MOVE_LINE_DOWN));
 
-	if (!text || !text->curl || !text->sell) return;
+	if (!text->curl || !text->sell) return;
 	
 	txt_order_cursors(text, false);
 
@@ -2934,6 +2983,7 @@ int txt_setcurr_tab_spaces(Text *text, int space)
 	const char *comm = "#";
 	const char indent = (text->flags & TXT_TABSTOSPACES) ? ' ' : '\t';
 	static const char *back_words[] = {"return", "break", "continue", "pass", "yield", NULL};
+
 	if (!text->curl) return 0;
 
 	while (text->curl->line[i] == indent) {
@@ -3008,37 +3058,37 @@ bool text_check_delim(const char ch)
 
 	for (a = 0; a < (sizeof(delims) - 1); a++) {
 		if (ch == delims[a])
-			return 1;
+			return true;
 	}
-	return 0;
+	return false;
 }
 
 bool text_check_digit(const char ch)
 {
-	if (ch < '0') return 0;
-	if (ch <= '9') return 1;
-	return 0;
+	if (ch < '0') return false;
+	if (ch <= '9') return true;
+	return false;
 }
 
 bool text_check_identifier(const char ch)
 {
-	if (ch < '0') return 0;
-	if (ch <= '9') return 1;
-	if (ch < 'A') return 0;
-	if (ch <= 'Z' || ch == '_') return 1;
-	if (ch < 'a') return 0;
-	if (ch <= 'z') return 1;
-	return 0;
+	if (ch < '0') return false;
+	if (ch <= '9') return true;
+	if (ch < 'A') return false;
+	if (ch <= 'Z' || ch == '_') return true;
+	if (ch < 'a') return false;
+	if (ch <= 'z') return true;
+	return false;
 }
 
 bool text_check_identifier_nodigit(const char ch)
 {
-	if (ch <= '9') return 0;
-	if (ch < 'A') return 0;
-	if (ch <= 'Z' || ch == '_') return 1;
-	if (ch < 'a') return 0;
-	if (ch <= 'z') return 1;
-	return 0;
+	if (ch <= '9') return false;
+	if (ch < 'A') return false;
+	if (ch <= 'Z' || ch == '_') return true;
+	if (ch < 'a') return false;
+	if (ch <= 'z') return true;
+	return false;
 }
 
 #ifndef WITH_PYTHON
@@ -3056,8 +3106,8 @@ int text_check_identifier_nodigit_unicode(const unsigned int ch)
 bool text_check_whitespace(const char ch)
 {
 	if (ch == ' ' || ch == '\t' || ch == '\r' || ch == '\n')
-		return 1;
-	return 0;
+		return true;
+	return false;
 }
 
 int text_find_identifier_start(const char *str, int i)
